#include <stdlib.h>
#include <math.h>
#include "ams.h"

/************************************************************

The Qlib program library has resulted from the work conducted
over several years jointly by

 Laboratory of Telecommunications Technology of
 Helsinki University of Technology (HUT) and

 VTT Information Technology, Telecommunications

The following individuals have contributed to the library:

 Samuli Aalto, Jani Lakkakorpi, Ilkka Norros, Anssi Pirhonen
 and Jorma Virtamo.

The programs can be used and modified freely.

No claims are made about the correctness of the programs
and no liability is taken for any damage caused by the use
of the programs in the library.

Reports of bugs in the program can be sent to qlib@tct.hut.fi

*************************************************************/

static void calc_z(AMS *);
static void calc_a(AMS *);

/* These functions calculate the queue length distribution for
   Anick-Mitra-Sondhi system. Notes mostly like in their paper. */

static double aq(double k, double N, double c) { 
  return sqr(N/2 - k) - sqr(N/2 - c);
}

static double bq(double k, double N, double c, double lambda) {
  return 2 * (1-lambda) * sqr(N/2 - k) - N*(1+lambda) * (N/2 - c);
}

static double cq(double k, double N, double lambda) {
  return -sqr(1+lambda)*(sqr(N/2)-sqr(N/2-k));
}


static void
calc_z(AMS *m) /* Calculates the eigenvalues to table m->eig_vals. */
{
  int i=0;
  double N = (double) m->size;
  double k, c = m->c;
  double *z = m->eig_vals;
  double lambda = m->lambda;
  double aqk, bqk, cqk;
  
  if (c < N/2)
    {                       
      for (k = 0; k <= c; k++)
	{
	  aqk = aq(k, N, c);
	  bqk = bq(k, N, c, lambda);
	  cqk = cq(k, N, lambda);
	  if (k != c)
	    z[i++]=(-bqk-sqrt(sqr(bqk)-4*aqk*cqk)) / (2*aqk);
	  else
	    z[i++] = -cqk / bqk;
	}
      if (2*(N/2) == N)
	z[i++]= -N*(1+lambda)/(N-2*c);
      for (k = c + 1; k < N/2 - 0.1; k++)
	{
	  aqk = aq(k, N, c);
	  bqk = bq(k, N, c, lambda);
	  cqk = cq(k, N, lambda);
	  z[i++] = (-bqk-sqrt(sqr(bqk)-4*aqk*cqk)) / (2*aqk);
	  z[i++] = (-bqk+sqrt(sqr(bqk)-4*aqk*cqk)) / (2*aqk);
	}
    }
  else
    for (k = 0; k < N - c; k++) {
      aqk = aq(k, N, c);
      bqk = bq(k, N, c, lambda);
      cqk = cq(k, N, lambda);
      z[i++]=(-bqk-sqrt(sqr(bqk)-4*aqk*cqk)) / (2*aqk);
    }
}

static void
calc_a(AMS *m) /* Calculates the logarithms of the coefficients to table a. */
{
  int i,j,sign;
  double x,c1,r1,r2;
  double N = (double) m->size;
  double c = m->c;
  double lambda = m->lambda;
  double *z = m->eig_vals;
  double *a = m->coeffs;
  char *signs = m->signs;
  
  for (j=0;j<N-(int)c;j++)
    {
      x=0;
      sign = -1;
      r1=(-(z[j]+1-lambda)+sqrt(sqr(z[j]+1-lambda)+4*lambda))
	/(2*lambda);
      r2=(-(z[j]+1-lambda)-sqrt(sqr(z[j]+1-lambda)+4*lambda))
	/(2*lambda);
      c1=(z[j]*c+N*lambda*(r1-1))/(lambda*(r1-r2));
      for (i=0;i<N-(int)c;i++)
	if (i != j)
	  {
	    x += log(-z[i])-log(fabs(z[i]-z[j]));
	    if (z[i]>z[j])
	      sign = -sign;
	  }
      x += N*log(lambda/(1+lambda));
      x += c1*log(r1-1)+(N-c1)*log(1-r2);
      if (2*((int)(c1+0.1)/2) < (int)(c1+0.1))
	sign = -sign;
      a[j]=x;
      signs[j]=sign;		
    }
}                                     

double 
GAMS(AMS *m, double x)
{
  double y=0.0;
  int i;
  
  for (i=0; i < m->size - m->c; i++)
    y -= m->signs[i]*exp(m->eig_vals[i]*x+m->coeffs[i]);
  return y;
}

double 
EvAMS (AMS *m)
{
  double y=0.0;
  int i;
  
  for (i=0; i < m->size - m->c; i++)
    y += m->signs[i]*exp(m->coeffs[i])/m->eig_vals[i];
  return y;
}

AMS *
initAMS(int N_init, double lambda_init, double c_init)
{
  AMS *m;
  
  m = malloc(sizeof(AMS));
  m->size = N_init;
  m->lambda = lambda_init;
  m->c = c_init;
  m->eig_vals = calloc(N_init - c_init, sizeof(double));
  m->coeffs  = calloc(N_init - c_init, sizeof(double));
  m->signs = calloc(N_init - c_init, sizeof(char));
  
  calc_z(m);
  calc_a(m);
  
  return m;
}

void
freeAMS(AMS *m)
{
  if (m) {
    if (m->coeffs)
      free(m->coeffs);
    if (m->eig_vals)
      free(m->eig_vals);
    if (m->signs)
      free(m->signs);
    free(m);
  }
}



