/* File: createnet.c
 *
 * Description: File createnet.c is a code file containing implementations
 * for functions to generate randomly two dimensional points and to create 
 * the network based on gabriel criterion.
 *
 * Author: Vesa Timonen <Vesa.Timonen@hut.fi>
 * Last modified: 17.4.2002
 */

#include "createnet.h"

/* Instrumental functions: */
static double getx(void *ptr, int pos);
static double gety(void *ptr, int pos);
static double dist(double *point1, double *point2);
static double *rand_point(double *point, const double scale, const int shape);
static int check_inbox(int *xsorted, int *ysorted,
		       const int x1, const int x2, 
		       const int y1, const int y2);
static double points_in_bb(double **data, int nbr_nodes, int *xsorted, 
			   int *ysorted,  const int x1, const int x2, 
			   const int y1, const int y2);
static void indexfind(const int *data, const int t1, const int t2, 
		      int *s1, int *s2);
/* End: Instrumental functions */


double **init_points(double **net_nodes, int nbr_nodes) {

  int i;

  net_nodes = (double**) malloc(nbr_nodes * sizeof(double*));
  for(i = 0; i < nbr_nodes; i++)
    net_nodes[i] = (double*) malloc(2 * sizeof(double));

  return net_nodes;
}


void free_points(double **net_nodes, int nbr_nodes) {

  int i;

  for(i = 0; i < nbr_nodes; i++)
    free(net_nodes[i]);
  free(net_nodes);
}


double **generate_points(double **net_nodes, int nbr_nodes, 
			 double scale, int shape) {

  int i;

  for(i = 0; i < nbr_nodes; i++)
    net_nodes[i] = rand_point(net_nodes[i], scale, shape);

  return net_nodes;
}


double **init_matrix(double **dist_matrix, int nbr_nodes) {

  int i;

  dist_matrix = (double**) malloc(nbr_nodes * sizeof(double*));
  for(i = 0; i < nbr_nodes; i++)
    dist_matrix[i] = (double*) malloc(nbr_nodes * sizeof(double));

  return dist_matrix;
}


void  free_matrix(double **dist_matrix, int nbr_nodes) {

  int i;

  for(i = 0; i < nbr_nodes; i++)
    free(dist_matrix[i]);
  free(dist_matrix);
}


double **create_connections(double **dist_matrix, double **net_nodes, 
			    int nbr_nodes) {

  TData connected_nodes = NULL;

  connected_nodes = init_Data(connected_nodes, 2 * nbr_nodes, 
			      (int)(nbr_nodes/2));
  connected_nodes = 
    create_connections_core(connected_nodes, net_nodes, nbr_nodes);

  dist_matrix = Data2double(dist_matrix, connected_nodes, nbr_nodes);

  free_Data(connected_nodes);
  return dist_matrix;
}


TData create_connections_core(TData c_nodes, double **net_nodes, 
			      int nbr_nodes) {

  int i, j;
  int ind1, ind2;
  int y1 = 0, y2 = 0;
  double dtemp = INF;

  int *xsorted;
  int *ysorted;

  /* allocate memory */
  xsorted = (int*) malloc(nbr_nodes * sizeof(int));
  ysorted = (int*) malloc(nbr_nodes * sizeof(int));

  /* init the index tables */
  for(i = 0; i< nbr_nodes; i++) {
    xsorted[i] = i;
    ysorted[i] = i;
  }

  /* get index tables sorted concerning  x and y coordinates */
  index_sort(xsorted, nbr_nodes, net_nodes, getx);
  index_sort(ysorted, nbr_nodes, net_nodes, gety);

  for(i = 0; i < nbr_nodes; i++) {

    ind1 = xsorted[i];

    for(j = i+1; j < nbr_nodes; j++){

      ind2 = xsorted[j];
      dtemp = INF;

      /* Find out the corresponding indices in the ysorted table */
      indexfind(ysorted, ind1, ind2, &y1, &y2);

      /* check if the inner box is empty */
      if(check_inbox(xsorted, ysorted, i, j, y1, y2) == 1) {
	
	/* check the bounding box and gabriel criterion inside the box */
	dtemp = points_in_bb(net_nodes, nbr_nodes, 
			     xsorted, ysorted, i, j, y1, y2);
      }

      /* if connection between nodes exists, add the data to c_nodes */
      if(dtemp != INF)
	set_Data(c_nodes, ind1, ind2, dtemp);
    }
  }

  /* set allocated memory free */
  free(xsorted);
  free(ysorted);

  optim_Data(c_nodes);
  return c_nodes;
}


/* Instrumental functions */
/* -----------------------*/

/* Function getx is an interface for using function index_sort. Getx
   returns x-coordinate of a point in given position in two dim. double 
   table.
   @param ptr           pointer to the table
   @param pos           position of the wanted point in the table
   @return              the x-coordinate of the point
*/
static double getx(void *ptr, int pos) {

  return ((double**)ptr)[pos][0];
}

/* Function gety is an interface for using function index_sort. Gety
   returns y-coordinate of a point in given position in two dim. double 
   table.
   @param ptr           pointer to the table
   @param pos           position of the wanted point in the table
   @return              the y-coordinate of the point
*/
static double gety(void *ptr, int pos) {

  return ((double**)ptr)[pos][1];
}

/* Function dist returns the distance between two points.
   @param point1        pointer to the first point
   @param point2        pointer to the second point
   @return              the distance between given two points
*/
static double dist(double *point1, double *point2) {

  double dx, dy;
  dx = point1[0] - point2[0];
  dy = point1[1] - point2[1];

  return sqrt(dx * dx + dy * dy);
}

/* Function rand_point generates randomly a two dimensional point
   @param point         pointer to the two dim point into which 
                        the generated data is set
   @param scale         scaling parameter, generated coordinates belong
                        to interval (0, scale)
   @param shape         shape of the net to be generated:
                        0 -> a circle
                        1 -> a box
   @return              pointer to the point
*/
static double *rand_point(double *point, const double scale, const int shape) {

  double r = RAND; /* drand48(); */
  double f = RAND;

  if(shape == 0) {
    point[0] = scale * sqrt(r) * cos(2 * M_PI * f);
    point[1] = scale * sqrt(r) * sin(2 * M_PI * f);
  }
  else {
    point[0] = scale * r;
    point[1] = scale * f;
  }
  return point;
}

/* Function check_inbox examines, if box bounded by points with indices
   x1/y1 and x2/y2 contain at least one point.
   @param xsorted       x-sorted index table
   @param ysorted       y-sorted index table
   @param x1            place of the point in the x-sorted table
   @param x2            place of the another point in the x-sorted table
   @param y1            place of the point in the y-sorted table
   @param y2            place of the another point in the y-sorted table
   @return              1, if the box is empty
                        0, if otherwise
 */
static int check_inbox(int *xsorted, int *ysorted,
		       const int x1, const int x2, 
		       const int y1, const int y2) {

  int i, j;
  int temp;

  if (x2 - x1 == 1) 
    return 1;
  
  if (y2 - y1 == 1)
    return 1;


  for(i = x1 + 1; i < x2; i++) {
    temp = xsorted[i];
    for(j = y1 + 1; j < y2; j++)
      if(ysorted[j] == temp)
	return 0;
  }
  
  return 1;
}

/* Function points_in_bb checks if the box containing circle bounded by 
   the points x1/y1 and x2/y2 (inbox excluded) contains points that do not
   fulfill the gabriel criterion.
   @param data          TVector containing the net nodes
   @param xsorted       x-sorted index table
   @param ysorted       y-sorted index table
   @param x1            place in the x-sorted table of the TPoint
   @param x2            place in the x-sorted table of the another TPoint
   @param y1            place in the y-sorted table of the TPoint
   @param y2            place in the y-sorted table of the another TPoint
   @return              distance between the points, if the gabriel criterion
                                                     is valid,
			INF, if otherwise
 */
static double points_in_bb(double **data, int nbr_nodes, int *xsorted, 
			   int *ysorted, const int x1, const int x2, 
			   const int y1, const int y2) {
  
  int i;
  double distance, t1, t2;
  double xmin, xmax, ymin, ymax;
  double *p1;
  double *p2; 
  double P[2];

  p1 = data[xsorted[x1]];
  p2 = data[xsorted[x2]];

  P[0] = (p1[0] + p2[0]) / 2;
  P[1] = (p1[1] + p2[1]) / 2;
  distance = dist(p1, p2);

  xmin = P[0] - distance / 2;
  xmax = xmin + distance;
  ymin = P[1] - distance / 2;
  ymax = ymin + distance;

  /* 1st critical area (on left) */

  t1 = data[ysorted[y1]][1];     /* lower y-bound */
  t2 = data[ysorted[y2]][1];     /* upper y-bound */

  for(i = x1 - 1; i >= 0; i--) {
    p1 = data[xsorted[i]];
    if(p1[0] >= xmin) {
      if( (p1[1] >= t1) && (p1[1] <= t2))
	if(dist(p1, P) < (distance / 2)) /* gabriel criterion */
	  return INF;
    }
    else i = -1;
  }
  
  /* 2nd critical area (on right) */

  for(i = x2 + 1; i < nbr_nodes; i++) {
    p1 = data[xsorted[i]];
    if(p1[0] <= xmax) {
      if( (p1[1] >= t1) && (p1[1] <= t2))
	if(dist(p1, P) < (distance / 2)) /* gabriel criterion */
	  return INF;
    }
    else i = nbr_nodes;
  }
  
  /* 3rd critical area (on bottom) */

  t1 = data[xsorted[x1]][0];     /* lower x-bound */
  t2 = data[xsorted[x2]][0];     /* upper x-bound */

  for(i = y1 - 1; i >= 0; i--) {
    p1 = data[ysorted[i]];
    if(p1[1] >= ymin) {
      if( (p1[0] >= t1) && (p1[0] <= t2))
	if(dist(p1, P) < (distance / 2)) /* gabriel criterion */
	  return INF;
    }
    else i = -1;
  }

  /* 4th critical area (on top) */

  for(i = y2 + 1; i < nbr_nodes; i++) {
    p1 = data[ysorted[i]];
    if(p1[1] <= ymax) {
      if( (p1[0] >= t1) && (p1[0] <= t2))
	if(dist(p1, P) < (distance / 2)) /* gabriel criterion */
	  return INF;
    }
    else i = nbr_nodes;
  }

  return distance;
}

/* Function indexfind searchs given two idices from given table and
   sets their positions in the table to given variables in increasing order.
   @param base   integer table
   @param t1     first element to be searched
   @param t2     second element to be searched
   @param s1     pointer to the variable into which the position of 
                 the smaller element is to be placed
   @param s2     pointer to the variable into which the position of 
                 the greater element is to be placed
 */
static void indexfind(const int *data, const int t1, const int t2, 
	       int *s1, int *s2) {

  int k = -1, y1 = -1, y2 = -1;

  while(y1 == -1) {
    k++;
    if(data[k] == t1 || data[k] == t2)
      y1 = k;
  }
  
  while(y2 == -1) {
    k++;
    if(data[k] == t1 || data[k] == t2)
      y2 = k;
  }
  
    *s1 = y1;
    *s2 = y2;
}

