/* File: createnet.h
 *
 * Description: File createnet.h is a header file that contains descriptions
 * and interfaces for functions to generate randomly two dimensional points
 * and to create the network based on gabriel criterion.
 *
 * Author: Vesa Timonen <Vesa.Timonen@hut.fi>
 * Last modified: 2.5.2002
 */


#ifndef _CREATENET_H
#define _CREATENET_H

#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "network_data.h"
#include "indexsort.h"

#ifndef INF
#define INF -1
#endif /* INF */

# ifndef M_PI
# define M_PI 3.141592653589793238462643383
# endif

#if __MSDOS__ || _Windows || __WIN32__
#define RAND (((double)rand())/RAND_MAX)
#define SRAND randomize()
#else
/* Following 12 rows by Jouni Karvo. ThankS! */
#ifdef drand48
#define RAND drand48()
#define SRAND srand48((long)time(NULL))
#else
#ifdef random
#define RAND (((double)random())/RAND_MAX)
#define SRAND srandom((unsigned int)time(NULL))
#else
#define RAND (((double)rand())/RAND_MAX)
#define SRAND srand((unsigned int)time(NULL))
#endif
#endif
#endif

/* Functions for point generation */
/* ------------------------------ */

/* Function init_points allocates memory for a variable into which the
   generated points will be placed.
   @param net_nodes     pointer to two dim. table to be initialized
   @param nbr_nodes     number of the points in the table
   @return              pointer to the initialized table
 */
double **init_points(double **net_nodes, int nbr_nodes);

/* Function free_points sets free the memory allocated
   by function init_points.
   @param nbr_nodes     number of the points in the table
   @param  net_nodes    pointer to the two dim. table
 */
void free_points(double **net_nodes, int nbr_nodes);

/* Function generate_points generates randomly a table containing
   two dim. points.
   @param net_nodes     pointer to the two dim. table into which the
                        generated data is placed
   @param nbr_nodes     number of nodes in the net to be generated
   @param scale         scaling parameter
   @param shape         shape of the net to be generated:
                        0 -> a circle
                        1 -> a box
   @return              two dimensional table containing the coordinates
                        of the generated points
 */
double **generate_points(double **net_nodes, int nbr_nodes,
			 double scale, int shape);

/* Functions for creating connections */
/* ---------------------------------- */

/* Function init_matrix allocates memory for a variable into which the
   distance/cost matrix will be placed.
   @param dist_matrix   pointer to the two dim. table to be initialized
   @param nbr_nodes     number of nodes in the net
   @return              pointer to the initialized table
 */
double **init_matrix(double **dist_matrix, int nbr_nodes);

/* Function free_points sets free the memory allocated
   by function init_matrix.
   @param  dist_matrix  pointer to the two dim. table (dist/cost matrix)
   @param nbr_nodes     number of nodes in the net
 */
void  free_matrix(double **dist_matrix, int nbr_nodes);

/* Function create_connections creates a distance/cost matrix based on
   the Gabriel criterion. Distance/cost -1 denotes no connection/infinite cost.
   @param dist_matrix   pointer to the two dim. table into which the
                        result matrix is placed
   @param net_nodes     two dimensional table containing the node coordinates
   @param nbr_nodes     number of nodes in the net
   @return              two dimensional table (matrix) containing the
                        distances/costs between nodes
 */
double **create_connections(double **dist_matrix, double **net_nodes,
			    int nbr_nodes);

/* Function create_connections_core creates a TData stuct that contains
   node pairs that are connected based on the Gabriel criterion and the
   distance/cost between each pair. Distance/cost -1 denotes no
   connection/infinite cost.
   @param c_node        pointer to the TData structure into which the
                        results are placed
   @param net_nodes     two dimensional table containing the node coordinates
   @param nbr_nodes     number of nodes in the net
   @return              TData struct containing the connected node pairs and
                        distances/costs between each pair
 */
TData create_connections_core(TData c_nodes, double **net_nodes,
			      int nbr_nodes);

#endif /* _CREATENET_H */




