/* File: dijkstra.c
 *
 * Description: File dijkstra.c contains functions to find out the minimum
 * distance/cost path from the given root node to a specific node or to all
 * other nodes in the given network.
 *
 * Assumptions: All given distances/costs in the given network are positive.
 * In network containing N>1 nodes, the nodes are indexed with indices starting
 * from 0 to N-1.
 *
 * Author: Vesa Timonen <Vesa.Timonen@hut.fi>
 * Last modified: 5.5.2002
 */

#include "dijkstra.h"

/* Instrumental functions: */
static int cond(TData result, int interf_size, int dest);
static int other(TData data, int node, int pos);
static double getd(void *ptr, int pos);
static void init_vars(TData c_nodes, int nbr_nodes, int **data,
		      int *nbr_ngbrs);
static void update_table(TData c_nodes, int **data, int *nbr_elems,
			 int source);
static void update_interface(TData routes, int *interface, int *interf_size,
			     int *nbr_connections);
static void remove_crossrefs(TData c_nodes, TData routes, int **data,
			     int *nbr_elems, int *iface, int *if_size,
			     int source);
static TData find_route(TData routes, int target);
static void set_node(int *vect, int *nbr_elems, int node);
static void remove_node(int *vect, int *nbr_elems, int node);
/* End: Instrumental functions */


int nbr_connections(TData routes) {
  return get_nbr(routes);
}

int *dijkstra_core(int *core, TData routes) {

  int i;

  for(i = 0; i < get_nbr(routes); i++)
    core[i] = get_n2(routes, i);

  return core;
}

int *dijkstra_prev(int *prev, TData routes) {

  int i;

  for(i = 0; i < get_nbr(routes); i++)
    prev[i] = get_n1(routes, i);

  return prev;
}

double *dijkstra_dist(double *dist, TData routes) {

  int i;

  for(i = 0; i < get_nbr(routes); i++)
    dist[i] = get_dist(routes, i);

  return dist;
}

TData dijkstra(TData routes, double **dist_matrix,
	       int nbr_nodes, int root, int dest) {

  TData c_nodes = NULL;

  c_nodes = init_Data(c_nodes, 2 * nbr_nodes, (int)(nbr_nodes/2));
  c_nodes = double2Data(c_nodes, dist_matrix, nbr_nodes);
  optim_Data(c_nodes);

  routes = dijkstra_func(routes, c_nodes, nbr_nodes, root, dest);
  free_Data(c_nodes);

  return routes;
}

TData dijkstra_func(TData routes, TData c_nodes,
		    int nbr_nodes, int root, int dest) {

  int i, index = 0, source = 0, snode = 0, edge = 0;
  int interf_size = 0;  /* number of nodes in the interface */
  double min_dist, temp;

  int *nbr_neighbourgs; /* number of neighbourgs of each node              */
  int *interface;       /* core node indices that have non-core node ngbrs */
  /* NOTA BENE! interface node is position index in the core list          */
  int **data_sorted;    /* sorted index table of connections of each node  */


  /* allocate memory */
  nbr_neighbourgs = (int*) malloc(nbr_nodes * sizeof(int));
  interface       = (int*) malloc(nbr_nodes * sizeof(int));
  data_sorted     = (int**) malloc(nbr_nodes * sizeof(int*));

  /* initialize sorted index table */
  init_vars(c_nodes, nbr_nodes, data_sorted, nbr_neighbourgs);
  update_table(c_nodes, data_sorted, nbr_neighbourgs, root);

  /* set root to routes and to table interface */
  set_Data(routes, root, root, 0);
  if (nbr_neighbourgs[root] != 0)
    set_node(interface, &interf_size, 0);
  else
    interf_size = 0;

  /* loop to find the shortest distances... */
  while (cond(routes, interf_size, dest) == 1) {

    index    = 0;
    source   = interface[0];
    edge     = data_sorted[get_n2(routes, source)][0];
    min_dist = get_dist(routes, source) + get_dist(c_nodes, edge);

    /* find the closest node from the neighbourgs of interface nodes */
    for (i = 1; i < interf_size; i++) {
      source = interface[i];
      edge   = data_sorted[get_n2(routes, source)][0];
      temp   = get_dist(routes, source) + get_dist(c_nodes, edge);
      if (temp < min_dist) {
	min_dist  = temp;
	index = i;
      }
    }

    source = interface[index];
    snode  = get_n2(routes, source);
    edge   = data_sorted[snode][0];
    index  = other(c_nodes, snode, edge);

    /* set new closest node data to results */
    set_Data(routes, snode, index, min_dist);

    /* update the lists */
    /*    update_table(data_sorted, nbr_nodes, nbr_neighbourgs, edge);*/
    remove_node(data_sorted[snode], nbr_neighbourgs + snode, edge);
    update_table(c_nodes, data_sorted, nbr_neighbourgs, index);

    remove_crossrefs(c_nodes, routes, data_sorted, nbr_neighbourgs,
		     interface, &interf_size, index);
    update_interface(routes, interface, &interf_size, nbr_neighbourgs);
  } /* end of the loop */

  /* set allocated memory free */
  for(i = 0; i < nbr_nodes; i++)
    free(data_sorted[i]);
  free(nbr_neighbourgs);
  free(data_sorted);
  free(interface);

  if (dest != -1) {
    if (get_n2(routes, get_nbr(routes) - 1) != dest)
      dest = root;
    routes = find_route(routes, dest);
  }

  optim_Data(routes);
  return routes;
}

/* Instrumental functions */
/* ---------------------- */

/* Function cond checks if there still is nodes to be found.
   @param result           TData containing found nodes
   @param interf_size      size of the interface vector
   @param dest             possible destination node (-1 if there is no
                                                      specific target)
   @return                 1, if there still are nodes to be found
                           -1, if otherwise
 */
static int cond(TData result, int interf_size, int dest) {

  if (interf_size == 0) /* no more neighbourgs */
    return -1;

  if (dest == -1) /* no specific destination */
    return 1;

  if (get_n2(result, get_nbr(result) - 1) == dest)
    return -1; /* destination found */

  return 1;
}

/* Function other returns the other node of a node pair in given position
   in given TData, when the other node is known...
   @param data             TData containing the nodes
   @param node             the other known node
   @param pos              position of the node pair in the data
   @return                 the other node in the node pair
 */
static int other(TData data, int node, int pos) {

  if (get_n1(data, pos) == node)
    return get_n2(data, pos);

  return get_n1(data, pos);
}

/* function getd forms a compatible interface for function
   get_dist (TData function) and can be used with function
   indexsort */
static double getd(void *ptr, int pos) {
  return get_dist((TData)ptr, pos);
}

/* Function init_vars initializes parameters.
   @param c_node           TData containing the connected node pairs
   @param nbr_nodes        number of nodes in the networ k
   @param data             two dim table
   @param nbr_ngbrs        one dim table
 */
static void init_vars(TData c_nodes, int nbr_nodes, int **data,
		      int *nbr_ngbrs) {

  int i, j, k = 0;
  int n1 = 0, n2 = 0;
  int *temp;

  temp = (int*) malloc(nbr_nodes * sizeof(int));

  for(i = 0; i < nbr_nodes; i++) {

    k = 0;
    for(j = 0; j < get_nbr(c_nodes); j++) {
      n1 = get_n1(c_nodes, j);
      n2 = get_n2(c_nodes, j);
      if (n1 == i || n2 == i) {
	temp[k] = j;
	k++;
      }
    }
    nbr_ngbrs[i] = k;

    if (k != 0) {
      data[i] = (int*) malloc(k * sizeof(int));
      for (k = 0; k < nbr_ngbrs[i]; k++)
	data[i][k] = temp[k];

      index_sort(data[i], nbr_ngbrs[i], c_nodes, getd);
    }
  }

  free(temp);
}

/* Function update_table updates the integer table containing the posible
   node pair indices still available...
 */
static void update_table(TData c_nodes, int **data, int *nbr_elems,
			 int source) {

  int i, dest = 0;

  for (i = 0; i < nbr_elems[source]; i++) {
    dest = other(c_nodes, source, data[source][i]);
    if(nbr_elems[dest] != 0)
      remove_node(data[dest], nbr_elems + dest, data[source][i]);
  }
}


/* Function update_interface updates the interface...
 */
static void update_interface(TData routes, int *interface, int *interf_size,
			     int *nbr_connections) {

  int i, pos = 0;

  /* remove nodes that don't have neighbourgs from the list interface */
  for(i = *interf_size - 1; i >= 0; i--)
    if (nbr_connections[get_n2(routes, interface[i])] == 0)
      remove_node(interface, interf_size, interface[i]);

  /* position of the last core node in the core list */
  pos = get_nbr(routes) - 1;

  /* Add the last found core node position to interface if
     it has neighbourgs */
  if (nbr_connections[get_n2(routes, pos)] > 0)
    set_node(interface, interf_size, pos);

}

/* Function remove_crossrefs
 */
static void remove_crossrefs(TData c_nodes, TData routes, int **data,
			     int *nbr_elems, int *iface, int *if_size,
			     int source) {

  int i, j;
  int  node = 0;

  for (i = 0; i < *if_size; i++) {
    node = get_n2(routes, iface[i]);
    for (j = nbr_elems[node] - 1; j >= 0; j--)
      if (other(c_nodes, node, data[node][j]) == source)
	remove_node(data[node], nbr_elems + node, data[node][j]);
  }
}


/* Function find_route removes non-relevant data i.e. nodes that do
   not belong to the route from root to target.
   @param routes        TData containing the path data
   @param target        the target node
   @return              the TData containing only the single path from
                        the root to the target
 */
static TData find_route(TData routes, int target) {

  TData temp = NULL;
  int i, node = 0;

  temp = init_Data(temp, get_nbr(routes), get_step(routes));
  node = target;

  for (i = get_nbr(routes) - 1; i >= 0; i--)
    if (node == get_n2(routes, i)) {
      set_Data(temp, get_n1(routes, i), node, get_dist(routes, i));
      node = get_n1(routes, i);
    }

  free_Data(routes);
  routes = init_Data(routes, get_nbr(temp), get_step(temp));

  for (i = get_nbr(temp) - 1; i >= 0; i--)
    set_Data(routes, get_n1(temp, i), get_n2(temp, i), get_dist(temp, i));

  free_Data(temp);
  return routes;
}


/* Function set_node sets a node to a given vector and increases its size
 */
static void set_node(int *vect, int *nbr_elems, int node) {

  vect[*nbr_elems] = node;
  *nbr_elems = *nbr_elems + 1;

}

/* Function remove_node removes a node from a given vector (if it contains
   the node to be removed) and decreases the size of the vector
 */
static void remove_node(int *vect, int *nbr_elems, int node) {

  int i, k = 0;

  /* the element to be removed is the last one */
  if (vect[*nbr_elems - 1] == node)
    *nbr_elems = *nbr_elems - 1;

  /* the element to be removed is not the last one... */
  /* find the position of the node */
  else {
    k = 0;

    while ((vect[k] != node) && (k < *nbr_elems - 1))
      k++;

    if(k < *nbr_elems - 1) {
      for (i = k; i < *nbr_elems - 1; i++)
	vect[i] = vect[i+1];
      *nbr_elems = *nbr_elems - 1;
    }
  }
}
