/* File: dijkstra.h
 *
 * Description: File dijkstra.h is a header file containing function
 * descriptions for file dijkstra.c. Dijkstra.h describes functions to 
 * find out the minimum distance/cost path from the given root node to 
 * a specific node or to all  other nodes in the given network.
 *
 * Assumptions: All given distances/costs in the given network are positive.
 * In network containing N>1 nodes, the nodes are indexed with indices starting
 * from 0 to N-1.
 * Each node is connected at least with one other node.
 *
 * Author: Vesa Timonen <Vesa.Timonen@hut.fi>
 * Last modified: 17.4.2002
 */

#ifndef _DIJKSTRA_H
#define _DIJKSTRA_H

#include <stdlib.h>
#include "network_data.h"
#include "indexsort.h"

#ifndef INF
#define INF -1
#endif /* INF */

/* Function nbr_connections returns the number of connected node pairs in
   the tree of sortest path.
   @param routes       TData object containing the sortest path tree
   @return             the number of connected node pairs
 */
int nbr_connections(TData routes);

/* Function dijkstra_core returns one dimensional table containing the core
   nodes of the sortest path tree.
   @param core         pointer to the integer table in which 
                       the result is placed
   @param routes       TData object containing the sortest path tree
   @return             pointer to one dim. table containing the core nodes
                       of the sortest path tree
 */
int *dijkstra_core(int *core, TData routes);

/* Function dijkstra_prev returns one dimensional table containing the
   parent nodes of corresponding nodes in the core node list.
   @param prev         pointer to the integer table in which 
                       the result is placed
   @param routes       TData object containing the sortest path tree
   @return             pointer to one dim. table containing the parent nodes
                       of the core nodes
 */
int *dijkstra_prev(int *prev, TData routes);

/* Function dijkstra_dist returns one dimensional table containing distances
   between root node and corresponding node in the table core.  
   @param dist         pointer to the double table in which 
                       the result is placed
   @param routes       TData object containing the sortest path tree
   @return             pointer to one dim. table containing the minimum
                       distances from root node to other nodes listed
		       in the table core
*/
double *dijkstra_dist(double *dist, TData routes);

/* Function dijkstra finds out the minimum distance/cost path from 
   the given root node to a specific node or to all other nodes in 
   the given network.
   @param routes       TData object in which the results are placed
   @param dist_matrix  two dimensional table containing the distaces/costs
                       between nodes
   @param nbr_nodes    the number of nodes in the network
   @param root         the root node
   @param dest         the destination node
                       if dest = -1, then the paths t all nodes in the network
		       are searched
   @return             TData object containing the sortest path/minimum cost
                       tree, in form of three tables: 
                         previous node table
			 core node table
			 distance/cost table
 */
TData dijkstra(TData routes, double **dist_matrix, 
		  int nbr_nodes, int root, int dest);

/* Function dijkstra_func finds out the minimum distance/cost path from 
   the given root node to a specific node or to all other nodes in 
   the given network.
   @param routes       TData object in which the results are placed
   @param c_nodes      TData object containing the connected node pairs in the
                       network and the distance/cost between nodes
   @param nbr_nodes    the number of nodes in the network
   @param root         the root node
   @param dest         the destination node
                       if dest = -1, then the paths t all nodes in the network
		       are searched
   @return             TData object containing the sortest path/minimum cost
                       tree, in form of three tables: 
                         previous node table
			 core node table
			 distance/cost table
 */
TData dijkstra_func(TData routes, TData c_nodes, 
		    int nbr_nodes, int root, int dest);

#endif /* _DIJKSTRA_H */
