/* File: network_data.h
 *
 * Description: File network_data.h is a header file that contains 
 * descriptions and interfaces for TData structure and for functions 
 * to handle it.
 *
 * Author: Vesa Timonen <Vesa.Timonen@hut.fi>
 * Last modified: 14.2.2002
 */

#ifndef _TDATA_H
#define _TDATA_H

#include <stdlib.h>

#ifndef INF
#define INF -1
#endif /* INF */

/* A struct definition for a data type TData, that contains two dimensional
   integer table and a one dimensional double table. TData also contains
   three integer variable for maintaining the struct. 
   max_nbr            maximum number of integer pairs and double numbers
                      that the tables can contain ie. number of allocated
		      places
   nbr_connections    number of the integer pairs and doubles that are already
                      in the tables
   step_size          number of places that are allocated if it is necessary
                      to increase the table sizes
   connections        two dim. integer table for the integer pairs
   distances          one dim. double table
 */
typedef struct {

  int max_nbr;
  int nbr_connections;
  int step_size;
  int **connections;
  double *distances;
} *TData;

/* Function init_Data initializes the TData variabe. It allocates the memory
   and sets the data to the maintenance variables.
   @param data          pointer to the TData variable to be inited
   @param maximum       maximum number of int. pairs and doubles to be
                        stored
   @param step          number of the places that are allocated if necessary
   @return              pointer to the inited TData variable
 */
TData init_Data(TData data, int maximum, int step);

/* Function free_Data sets free the menory allocated for the given TData.
   @param data         TData to be set free
*/
void free_Data(TData data);

/* Function set_Data sets three elements to given TData.
   @param data         TData in which the data is to be set.
   @param node1        integer to be set into the first integer table
   @param node2        integer to be set into the second integer table
   @param dist         double to be set into the double table
*/
void set_Data(TData data, int node1, int node2, double dist);

/* Function optim_Data optimizes the given TData, i.e. deallocates memory
   that is not needed.
   @param data         TData to be optimized
*/
void optim_Data(TData data);

/* Function get_max returns the current maximum length of the data tables 
   in the given TData.
   @param data         TData which is in focus...
   @return             the maximum size of the data vectors
*/
int get_max(TData data);

/* Function get_nbr returns the current length of the data vector.
   @param data         TData which is in focus...
   @return             the current size of the data vectors
*/
int get_nbr(TData data);

/* Function get_step returns the step size with which the maximum size
   is increased if the data vectors get full.
   @param data         TData which is in focus...
   @return             the current step size
*/
int get_step(TData data);

/* Function get_n1 returns the data in the first integer vector in given
   position in given TData.
   @param data         TData which is in focus...
   @param pos          position of the wanted node
   @return             the node in the given position in the first table
*/
int get_n1(TData data, int pos);

/* Function get_n2 returns the data in the second integer vector in given
   position in given TData.
   @param data         TData which is in focus...
   @param pos          position of the wanted node
   @return             the node in the given position in the second table
*/
int get_n2(TData data, int pos);

/* Function get_dist returns the data in the third vector in given
   position in given TData.
   @param data         TData which is in focus...
   @param pos          position of the wanted node
   @return             the node in the given position in the third table
*/

double get_dist(TData data, int pos);

/* Function Data2double converts information of TData to a distance matrix
   @param target       pointer to two dim. double table into which the
                       distance matrix is placed
   @param source       TData containing the information
   @param nbr_nodes    number of nodes in the network
   @return             pointer to the distance matrix
*/
double **Data2double(double **target, TData source, int nbr_nodes);

/* Function double2Data converts distance matrix to a TData.
   @param target       TData into which the network data is placed
   @param source       pointer to the distance matrix
   @param nbr_nodes    number of nodes in the network
   @return             TData containing the network data
*/
TData double2Data(TData target, double **source, int nbr_nodes);

#endif /* _TDATA_H */

