
/* File: qlib.c
 *
 * These functions merely serve as interface between
 * traffic library qlib.a written in C
 * and Mathematica
 * __
 * UpDates:
 * 18.04.2002 lnkcreatenet and lnkdijkstra added by Vesa
 * 27.05.2002 lnkcreatenet renamed as lnkgabrielnet
 */

#include <math.h>
#include <string.h>
#include "erlang.h"
#include "queuel.h"
#include "mathlink.h"
#include "qlib.h"
#include "network_data.h"
#include "createnet.h"
#include "dijkstra.h"

/* local functions */
double D(double x) { return  x < 1 ? 0 : 1; }
double M(double x) { return  1.0 - exp(-x); }

void lnkMg1(int x, double rho)
{
    int     i=0;
    double *result;

    MLGetInteger(stdlink, &i);

    switch( i )
    {
        case 1:
            result = Mg1(x, rho, D);
            MLPutRealList(stdlink, result, x+1);
            break;
        case 2:
            result = Mg1(x, rho, M);
            MLPutRealList(stdlink, result, x+1);
        default:
            MLEvaluate(stdlink, "Message[LnkQmg1::usage]");
            break;
    }
}


void lnkSumMd1(int x, double rho)
{
    double *result;

    result = SumMd1(x, rho);
    MLPutRealList(stdlink, result, x+1);
}


void lnkRecMd1(int x, double rho)
{
    double *result;

    result = RecMd1(x, rho);
    MLPutRealList(stdlink, result, x+1);
}


void lnkMdn(double x, double rho, int n)
{
    double *result;

    result = Mdn(x, rho, n);
    MLPutRealList(stdlink, result, x+1);
}


double lnkbmitra(int j, int J, double C, int *d, long dl,
                 double *a, long al)
{
    if (al == dl && al == J)
        return Bmitra(j, J, C, d, a);
    else
        return 0;
}


double lnkbkaufman(int k, int K, int C, int *dk, long dl, double *ak, long al)
{
    if (al == dl && al == K)
        return Bkaufman(k, K, C, dk, ak);
    else
        return  0;
}

/* Function lnkgabrielnet generates a random network containing given number of nodes.
   Node pair connection is based on the Gabriel criterion. Function returns via MathLink
   a list containing two elements:
     1. list containing the coordinates of generated points
     2. distance/cost matrix, in which -1 denotes no connection/infinite cost.
   @param nbr_nodes     number of nodes in the net to be generated
   @param scale         scaling parameter
   @param shape         shape of the net to be generated:
                        0 -> a circle
                        1 -> a box
 */
void lnkgabrielnet (const int nbr_nodes, const double scale, const int shape)
{

  int i;
  double **points;
  double **net;

  points = init_points(points, nbr_nodes);
  net    = init_matrix(net, nbr_nodes);

  points = generate_points(points, nbr_nodes, scale, shape);
  net    = create_connections(net, points, nbr_nodes);

  MLPutFunction (stdlink, "List", 2);
  /* return the coordinates of the generated points */
   MLPutFunction (stdlink, "List", nbr_nodes);
   for (i = 0; i < nbr_nodes; i++)
     MLPutRealList(stdlink, points[i], 2);
  /* return the distance matrix */
  MLPutFunction (stdlink, "List", nbr_nodes);
   for (i = 0; i < nbr_nodes; i++)
     MLPutRealList(stdlink, net[i], nbr_nodes);

  free_matrix(net, nbr_nodes);
  free_points(points, nbr_nodes);
}


/* Function lnkdijkstra finds out the minimum distance/cost path from
   the given root node to a specific node or to all other nodes in
   the given network. The distance/cost matrix is read manually via MathLink.
   Function returns via MathLink a list containing three elements:
    1. a list of core nodes
    2. a list of previous nodes
    3. a list of distances
   @param root         the root node
   @param target       the destination node
                       if target = -1, then the paths t all nodes in the network
                       are searched
 */
void lnkdijkstra (const int root, const int target)
{

  int i, j,
    nbr_nodes   = 0,
    connections = 0;
  long temp     = 0;
  int *nodes;
  double *dists;
  double **distances;
  TData routes;

  /* check the distance matrix size */
  MLCheckFunction(stdlink, "List", &temp);
  nbr_nodes = (int)temp;

  /* allocate memory for the distance matrix */
  distances = (double**)malloc(nbr_nodes * sizeof(double*));
  for (i = 0; i < nbr_nodes; i++)
    distances[i] = (double*)malloc(nbr_nodes * sizeof(double));

  /* read 'manually' the distance matrix from the MMA */
  for (i = 0; i < nbr_nodes; i++) {
    MLCheckFunction(stdlink, "List", &temp);
    for (j = 0; j < nbr_nodes; j++)
      MLGetDouble(stdlink, &(distances[i][j]));
  }

  routes = init_Data(routes, nbr_nodes, (int)(nbr_nodes / 2));

  routes = dijkstra(routes, distances, nbr_nodes, root, target);
  connections = nbr_connections(routes);

  nodes = (int*)malloc(connections * sizeof(int));
  dists = (double*)malloc(connections * sizeof(double));

  nodes = dijkstra_core(nodes, routes);
  dists = dijkstra_dist(dists, routes);

  /* return results to Mathematica */
  MLPutFunction (stdlink, "List", 3);
    MLPutIntegerList(stdlink, nodes, connections);
  nodes = dijkstra_prev(nodes, routes);
    MLPutIntegerList(stdlink, nodes, connections);
    MLPutRealList(stdlink, dists, connections);

    /* set free the allocated memory */
  free(nodes);
  free(dists);
  for (i = 0; i < nbr_nodes; i++)
    free(distances[i]);
  free(distances);
  free_Data(routes);
}

/*
 * Main loop, jump to Mathlink code
 */

#if MACINTOSH_MATHLINK

int main( int argc, char* argv[])
{
	/* Due to a bug in some standard C libraries that have shipped with
	 * MPW, zero is passed to MLMain below.  (If you build this program
	 * as an MPW tool, you can change the zero to argc.)
	 */

    SRAND; /* seed the rand, def. in createnet.h */

	argc = argc; /* suppress warning */
	return MLMain( 0, argv);
}

#elif WINDOWS_MATHLINK

#if __BORLANDC__
#pragma argsused
#endif

int PASCAL WinMain( HINSTANCE hinstCurrent, HINSTANCE hinstPrevious, LPSTR lpszCmdLine, int nCmdShow)
{
	char  buff[512];
	char FAR * buff_start = buff;
	char FAR * argv[32];
	char FAR * FAR * argv_end = argv + 32;

	hinstPrevious = hinstPrevious; /* suppress warning */

    SRAND; /* seed the rand, def. in createnet.h */

	if( !MLInitializeIcon( hinstCurrent, nCmdShow)) return 1;
	MLScanString( argv, &argv_end, &lpszCmdLine, &buff_start);
	return MLMain( argv_end - argv, argv);
}

#else

int main(argc, argv)
	int argc; char* argv[];
{
    SRAND; /* seed the rand, def. in createnet.h */
	return MLMain(argc, argv);
}

#endif
